package com.iainlobb.bunnylandmark 
{
	import com.iainlobb.gamepad.Gamepad;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	/**
	 * ...
	 * @author Iain Lobb - iainlobb@googlemail.com
	 */
	
	[SWF(width=640, height=480, backgroundColor=0xFFFFFF, frameRate=30)]
	 
	public class BitmapTest extends Sprite
	{
		public var numBunnies:int = 22000;
		public var gravity:Number = 1;
		public var bunnies:Vector.<BitmapBunny>;
		public var maxX:int;
		public var minX:int;
		public var maxY:int;
		public var minY:int;
		public var maxZ:int = 0;
		public var minZ:int = -10000;
		public var screenWidth:int = 640;
		public var screenHeight:int = 480;
		public var cameraSpeedX:Number = 4;
		public var cameraSpeedY:Number = 5;
		public var cameraRect:Rectangle;
		public var columns:int = 127;
		public var rows:int = 127;
		public var ground:Bitmap;
		public var tileSize:int = 32;
		public var gamepad:Gamepad;
		public var fps:FPS;
		
		[Embed(source = "/wabbit_alpha.png")]
		public var BunnyAsset:Class;
		
		[Embed(source = "/Outdoor_01.png")]
		public var GrassAsset:Class;
		
		public function BitmapTest() 
		{
			// CAMERA AND BOUNDRIES
			
			cameraRect = new Rectangle(0, 0, screenWidth, screenHeight);
			
			maxX = columns * tileSize;
			maxY = rows * tileSize;
			
			// GROUND
			
			ground = new Bitmap(new BitmapData(columns * tileSize, rows * tileSize));
			
			ground.bitmapData.lock();
			
			var grassAsset:Bitmap = new GrassAsset();
			var grassBitmapData:BitmapData = grassAsset.bitmapData;
			var sourceRect:Rectangle = new Rectangle(0, 0, 32, 32);
			var destPoint:Point = new Point();
			
			for (var row:int = 0; row < rows; row++)
			{
				for (var column:int = 0; column < columns; column++)
				{
					destPoint.x = column * tileSize;
					destPoint.y = row * tileSize;
					ground.bitmapData.copyPixels(grassBitmapData, sourceRect, destPoint); 
				}
			}
			
			ground.bitmapData.unlock();
			
			addChild(ground);
			
			// BUNNIES
			
			bunnies = new Vector.<BitmapBunny>();
			
			var bunnyAsset:Bitmap = new BunnyAsset();
			var bunny:BitmapBunny; 
			
			var bunnyRows:int = Math.sqrt(numBunnies);
			var spacingX:Number = maxX / bunnyRows;
			var spacingY:Number = maxY / bunnyRows;
			
			for (var i:int = 0; i < numBunnies; i++)
			{
				column = i % bunnyRows;
				row = i / bunnyRows;
				
				bunny = new BitmapBunny();
				bunny.bitmapData = bunnyAsset.bitmapData;
				bunny.positionX = column * spacingX;
				bunny.positionY = row * spacingY;
				
				bunny.angle = Math.random() * Math.PI * 2;
				bunny.speed = (Math.random() * 5) + 0.5;
				bunny.speedX = Math.sin(bunny.angle) * bunny.speed;
				bunny.speedY = Math.cos(bunny.angle) * bunny.speed;
				bunny.speedZ = -1;
				
				//addChild(bunny);
				bunnies.push(bunny);
			}
			
			// DISPLAY FRAMES PER SECOND AND MEMORY
			
			fps = new FPS();
			addChild(fps);
			
			// LISTEN FOR EVENTS
			
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
		}
		
		protected function onAddedToStage(event:Event):void
		{
			// CREATE GAMEPAD TO SCROLL CAMERA
			
			gamepad = new Gamepad(stage, true);
			gamepad.useWASD();
		}
		
		protected function onEnterFrame(event:Event):void
		{
			// CAMERA MOVMENT
			
			cameraSpeedX = gamepad.x * 10;
			cameraSpeedY = gamepad.y * 10;
			
			cameraRect.x += cameraSpeedX;
			cameraRect.y += cameraSpeedY;
			
			if (cameraRect.x > maxX - screenWidth)
			{
				cameraRect.x = maxX - screenWidth
				cameraSpeedX *= -1;
			}
			else if (cameraRect.x < 0)
			{
				cameraRect.x = 0;
				cameraSpeedX *= -1;
			}
			
			if (cameraRect.y > maxY - screenHeight)
			{
				cameraRect.y = maxY - screenHeight
				cameraSpeedY *= -1;
			}
			else if (cameraRect.y < 0)
			{
				cameraRect.y = 0;
				cameraSpeedY *= -1;
			}
			
			// BUNNIES
			
			var sourceRect:Rectangle = new Rectangle(0, 0, 26, 37);
			var bunny:BitmapBunny;
			var drawableBunnies:Vector.<BitmapBunny> = new Vector.<BitmapBunny>;
			
			for (var i:int = 0; i < numBunnies; i++)
			{
				bunny = bunnies[i];
				bunny.positionX += bunny.speedX;
				bunny.positionY += bunny.speedY;
				
				// POSITION X (horizontal position)
				
				if (bunny.positionX > maxX)
				{
					bunny.speedX *= -1;
					bunny.positionX = maxX;
				}
				else if (bunny.positionX < minX)
				{
					bunny.speedX *= -1;
					bunny.positionX = minX;
				}
				
				// POSITION Y (Depth into scene)
				
				if (bunny.positionY > maxY)
				{
					bunny.speedY *= -1;
					bunny.positionY = maxY;
				}
				else if (bunny.positionY < minY)
				{
					bunny.speedY *= -1;
					bunny.positionY = minY;
				}
				
				// POSITION Z (height off ground)
				
				bunny.positionZ += bunny.speedZ;
				bunny.speedZ += gravity;
				
				if (bunny.positionZ > maxZ)
				{
					bunny.speedZ *= -0.5;
					bunny.positionZ = maxZ;
					
					if (Math.random() > 0.1)
					{
						bunny.speedZ -= Math.random() * 1;
					}
				} 
				else if (bunny.positionZ < minZ)
				{
					bunny.speedZ = 0;
					bunny.positionZ = minZ;
				}
				
				// DRAW BUNNIES
				
				bunny.x = bunny.positionX - cameraRect.x;
				bunny.y = bunny.positionY + bunny.positionZ - cameraRect.y;
				
				if (bunny.x <= screenWidth && bunny.x > -sourceRect.width)
				{
					if (bunny.y <= screenHeight && bunny.y > -sourceRect.height)
					{
						drawableBunnies.push(bunny);
					}
				}
				
				if (!bunny.hidden) removeChild(bunny);
				bunny.hidden = true;
			}
			
			drawableBunnies.sort(sortDepths);
			var numDrawableBunnies:int = drawableBunnies.length;
			
			for (var i:int = 0; i < numDrawableBunnies; i++)
			{
				bunny = drawableBunnies[i];
				addChild(bunny);
				bunny.hidden = false;
			}
			
			// THIS TRACES AROUND 450
			// trace("numDrawableBunnies", numDrawableBunnies);
			
			ground.x = -cameraRect.x;
			ground.y = -cameraRect.y;
			
			setChildIndex(ground, 0);
			setChildIndex(fps, numChildren - 1);
		}
		
		private function sortDepths(bunny1:BitmapBunny, bunny2:BitmapBunny):int
		{
			if (bunny1.y < bunny2.y ) return -1;
			if (bunny1.y > bunny2.y) return 1;
			return 0;
		}
	}
}

