package com.iainlobb.bunnylandmark 
{
	import com.iainlobb.gamepad.Gamepad;
	import flash.display.Bitmap;
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.geom.Point;
	import flash.geom.Rectangle;
	/**
	 * ...
	 * @author Iain Lobb - iainlobb@googlemail.com
	 */
	
	[SWF(width=640, height=480, backgroundColor=0xFFFFFF, frameRate=30)]
	 
	public class BlitTest extends Sprite
	{
		public var numBunnies:int = 90000;
		public var gravity:Number = 1;
		public var bunnies:Vector.<BlitBunny>;
		public var maxX:int;
		public var minX:int;
		public var maxY:int;
		public var minY:int;
		public var maxZ:int = 0;
		public var minZ:int = -10000;
		public var screenWidth:int = 640;
		public var screenHeight:int = 480;
		public var bitmap:Bitmap;
		public var cameraSpeedX:Number = 4;
		public var cameraSpeedY:Number = 5;
		public var cameraRect:Rectangle;
		public var columns:int = 127;
		public var rows:int = 127;
		public var ground:BitmapData;
		public var tileSize:int = 32;
		public var gamepad:Gamepad;
		
		[Embed(source = "/wabbit_alpha.png")]
		public var BunnyAsset:Class;
		
		[Embed(source = "/Outdoor_01.png")]
		public var GrassAsset:Class;
		
		public function BlitTest() 
		{
			// CAMERA AND BOUNDRIES
			
			cameraRect = new Rectangle(0, 0, screenWidth, screenHeight);
			
			maxX = columns * tileSize;
			maxY = rows * tileSize;
			
			// GROUND
			
			ground = new BitmapData(columns * tileSize, rows * tileSize);
			
			ground.lock();
			
			var grassAsset:Bitmap = new GrassAsset();
			var grassBitmapData:BitmapData = grassAsset.bitmapData;
			var sourceRect:Rectangle = new Rectangle(0, 0, 32, 32);
			var destPoint:Point = new Point();
			
			for (var row:int = 0; row < rows; row++)
			{
				for (var column:int = 0; column < columns; column++)
				{
					destPoint.x = column * tileSize;
					destPoint.y = row * tileSize;
					ground.copyPixels(grassBitmapData, sourceRect, destPoint); 
				}
			}
			
			ground.unlock();
			
			// BUNNIES
			
			bunnies = new Vector.<BlitBunny>(numBunnies, true);
			
			var bunnyAsset:Bitmap = new BunnyAsset();
			var bunny:BlitBunny; 
			
			var bunnyRows:int = Math.sqrt(numBunnies);
			var spacingX:Number = maxX / bunnyRows;
			var spacingY:Number = maxY / bunnyRows;
			
			for (var i:int = 0; i < numBunnies; i++)
			{
				column = i % bunnyRows;
				row = i / bunnyRows;
				
				bunny = new BlitBunny();
				bunny.bitmapData = bunnyAsset.bitmapData;
				bunny.x = column * spacingX;
				bunny.y = row * spacingY;
				
				bunny.angle = Math.random() * Math.PI * 2;
				bunny.speed = (Math.random() * 5) + 0.5;
				bunny.speedX = Math.sin(bunny.angle) * bunny.speed;
				bunny.speedY = Math.cos(bunny.angle) * bunny.speed;
				bunny.speedZ = -1;
				
				bunnies[i] = bunny;
			}
			
			// "VIEWPORT"
			
			bitmap = new Bitmap(new BitmapData(maxX, maxY));
			addChild(bitmap);
			
			// DISPLAY FRAMES PER SECOND AND MEMORY
			
			addChild(new FPS());
			
			// LISTEN FOR EVENTS
			
			addEventListener(Event.ENTER_FRAME, onEnterFrame);
			addEventListener(Event.ADDED_TO_STAGE, onAddedToStage);
		}
		
		protected function onAddedToStage(event:Event):void
		{
			// CREATE GAMEPAD TO SCROLL CAMERA
			
			gamepad = new Gamepad(stage, true);
			gamepad.useWASD();
		}
		
		protected function onEnterFrame(event:Event):void
		{
			bitmap.bitmapData.lock();
			
			// CAMERA MOVMENT
			
			cameraSpeedX = gamepad.x * 10;
			cameraSpeedY = gamepad.y * 10;
			
			cameraRect.x += cameraSpeedX;
			cameraRect.y += cameraSpeedY;
			
			if (cameraRect.x > maxX - screenWidth)
			{
				cameraRect.x = maxX - screenWidth
				cameraSpeedX *= -1;
			}
			else if (cameraRect.x < 0)
			{
				cameraRect.x = 0;
				cameraSpeedX *= -1;
			}
			
			if (cameraRect.y > maxY - screenHeight)
			{
				cameraRect.y = maxY - screenHeight
				cameraSpeedY *= -1;
			}
			else if (cameraRect.y < 0)
			{
				cameraRect.y = 0;
				cameraSpeedY *= -1;
			}
			
			// GROUND
			
			bitmap.bitmapData.copyPixels(ground, cameraRect, new Point());  
			
			// BUNNIES
			
			var sourceRect:Rectangle = new Rectangle(0, 0, 26, 37);
			var bunny:BlitBunny;
			var drawableBunnies:Vector.<BlitBunny> = new Vector.<BlitBunny>;
			
			for (var i:int = 0; i < numBunnies; i++)
			{
				bunny = bunnies[i];
				bunny.x += bunny.speedX;
				bunny.y += bunny.speedY;
				
				// POSITION X (horizontal position)
				
				if (bunny.x > maxX)
				{
					bunny.speedX *= -1;
					bunny.x = maxX;
				}
				else if (bunny.x < minX)
				{
					bunny.speedX *= -1;
					bunny.x = minX;
				}
				
				// POSITION Y (Depth into scene)
				
				if (bunny.y > maxY)
				{
					bunny.speedY *= -1;
					bunny.y = maxY;
				}
				else if (bunny.y < minY)
				{
					bunny.speedY *= -1;
					bunny.y = minY;
				}
				
				// POSITION Z (height off ground)
				
				bunny.z += bunny.speedZ;
				bunny.speedZ += gravity;
				
				if (bunny.z > maxZ)
				{
					bunny.speedZ *= -0.5;
					bunny.z = maxZ;
					
					if (Math.random() > 0.1)
					{
						bunny.speedZ -= Math.random() * 1;
					}
				} 
				else if (bunny.z < minZ)
				{
					bunny.speedZ = 0;
					bunny.z = minZ;
				}
				
				// DRAW BUNNIES
				
				var drawPosition:Point = bunny.drawPosition;
				
				drawPosition.x = bunny.x - cameraRect.x;
				drawPosition.y = bunny.y + bunny.z - cameraRect.y;
				
				if (drawPosition.x <= screenWidth && drawPosition.x > -sourceRect.width)
				{
					if (drawPosition.y <= screenHeight && drawPosition.y > -sourceRect.height)
					{
						drawableBunnies.push(bunny);
					}
				}
			}
			
			drawableBunnies.sort(sortDepths);
			var numDrawableBunnies:int = drawableBunnies.length;
			
			for (var i:int = 0; i < numDrawableBunnies; i++)
			{
				bunny = drawableBunnies[i];
				bitmap.bitmapData.copyPixels(bunny.bitmapData, sourceRect, bunny.drawPosition, null, null, true);
			}
			
			// THIS TRACES AROUND 1900
			//trace("numDrawableBunnies", numDrawableBunnies);
			
			bitmap.bitmapData.unlock();
		}
		
		private function sortDepths(bunny1:BlitBunny, bunny2:BlitBunny):int
		{
			if (bunny1.y < bunny2.y ) return -1;
			if (bunny1.y > bunny2.y) return 1;
			return 0;
		}
	}
}

